#include "LE9x0Command.h"
#include <process.h>

// SOF
#define SOF_CMD				0xAA	// Command
#define SOF_RES				0x55	// Response

typedef enum {
	FRM_RECV_SOF = 0,
	FRM_RECV_CMD_CODE,
	FRM_RECV_SUB_RES_CODE,
	FRM_RECV_LENGTH_H,
	FRM_RECV_LENGTH_L,
	FRM_RECV_DATA,
	FRM_RECV_CHECKSUM,
	FRM_RECV_COMPLETE,
	FRM_RECV_NUM
} FRM_RECV_STEP;

static unsigned __stdcall ThreadFunc( void *p )
{
	LE9x0Command *pp = (LE9x0Command*)p;
	return pp->AnalizeFrameData();
}

LE9x0Command::LE9x0Command( CComm *comm ):
	mComm(comm)
{
	mHThread = NULL;
	InitializeCriticalSection( &mCriticalSection );

	// Create response list
	AddResponseList( CMD_CONNECT );
	AddResponseList( CMD_DISCONNECT );
	AddResponseList( CMD_DIN_GET );
	AddResponseList( CMD_AD_SPS_SET );
	AddResponseList( CMD_AD_RANGE_SET );
	AddResponseList( CMD_AD_CYCLE_SET );
	AddResponseList( CMD_AD_VALUE_GET );
	AddResponseList( CMD_AD_RUN );
	AddResponseList( CMD_AD_STOP );
	AddResponseList( CMD_DA_OUT_SET );
	AddResponseList( CMD_DA_OUT_GET );
	AddResponseList( CMD_DA_SWEEP_SET );
	AddResponseList( CMD_TEMP_SET );

	// Create response list
	AddCommandList( CMD_AD_RUN_RESULT );
	AddCommandList( CMD_AD_STOP_RESULT );
	AddCommandList( CMD_AD_TRANSFER_DATA );

	// Create a analize thread
	fTerminate = false;
	mHThread = (HANDLE)_beginthreadex(
		NULL,
		0,
		ThreadFunc,
		this,
		0,
		(unsigned int*)&mThreadId
	);
}

LE9x0Command::~LE9x0Command()
{
	DeleteCriticalSection( &mCriticalSection );
}

// Assemble frame data from receive buffer data.
// The parsed frame is stored in the command / response list.
DWORD LE9x0Command::AnalizeFrameData()
{
	static FRM_RECV_STEP step = FRM_RECV_SOF;
	static FRAME recv;
	static DWORD length_wk;
	BYTE data, buff[1024], *pbuff;
	DWORD buff_length;

	while( !fTerminate )
	{
		buff_length = mComm->GetReceiveData( buff );
		pbuff = buff;

		while( buff_length-- != 0 )
		{
			data = *(pbuff++);

			switch( step )
			{
			// SOF
			case FRM_RECV_SOF:
				if( (data == SOF_CMD) || (data == SOF_RES) )
				{
					recv.sof = data;
					recv.sum = 0;
					step = FRM_RECV_CMD_CODE;
				}
				break;
			// Command code
			case FRM_RECV_CMD_CODE:
				recv.cmd = data;
				step = FRM_RECV_SUB_RES_CODE;
				break;
			// Sub command code
			case FRM_RECV_SUB_RES_CODE:
				recv.sub = data;
				step = FRM_RECV_LENGTH_H;
				break;
			// Data length (High byte)
			case FRM_RECV_LENGTH_H:
				recv.length = (DWORD)data << 8;
				step = FRM_RECV_LENGTH_L;
				break;
			// Data length (Low byte)
			case FRM_RECV_LENGTH_L:
				recv.length |= (DWORD)data;
				length_wk = 0;
				if( recv.length > 0 )
				{
					step = FRM_RECV_DATA;
				}
				else
				{
					step = FRM_RECV_CHECKSUM;
				}
				break;
			// Data
			case FRM_RECV_DATA:
				if( length_wk < sizeof(recv.data) )
				{
					recv.data[length_wk++] = data;
					if( length_wk == recv.length )
					{
						step = FRM_RECV_CHECKSUM;
					}
				}
				break;
			// Checksum
			case FRM_RECV_CHECKSUM:
				recv.sum += 1;
				if( recv.sum == data )
				{
					if( recv.sof == SOF_CMD )
					{
						// Set command list
						SetCommandFrame( recv.cmd, &recv );
					}
					else if( recv.sof == SOF_RES )
					{
						// Set response list
						SetResponseFrame( recv.cmd, &recv );
					}
				}
				step = FRM_RECV_SOF;
				break;
			default:
				step = FRM_RECV_SOF;
				break;
			}

			recv.sum += data;
		}
		Sleep( 10 );
	}

	return S_OK;
}

// Send command or response.
// The checksum is calculated automatically.
// @param[in]	sof : Start of frame(0x55 or 0xAA)
// @param[in]	cmd : Comanndo code
// @param[in]	subres : Sub comannd code or Response code
// @param[in]	length : Frame data length
// @param[in]	pdata : Transmission data
void LE9x0Command::TransmitCommandResponse( BYTE sof, BYTE cmd, BYTE subres, DWORD length, BYTE* pdata )
{
	BYTE sum, sendbuff[1024], *pBuff = sendbuff;

	// Create frame data & Calculate checksum
	*pBuff = sof;					sum = *pBuff++;
	*pBuff = cmd;					sum += *pBuff++;
	*pBuff = subres;				sum += *pBuff++;
	*pBuff = (BYTE)(length >> 8);	sum += *pBuff++;
	*pBuff = (BYTE)(length);		sum += *pBuff++;
	if( (length >= 0) && (pdata != NULL) )
	{
		for( DWORD i = 0; i < length; i++ )
		{
			*pBuff = *(pdata + i);		sum += *pBuff++;
		}
	}
	*pBuff = sum + 1;

	// Transmit frame
	mComm->Transmit( sendbuff, length + 6 );
}

// Send command.
void LE9x0Command::TransmitCommand( BYTE cmd, BYTE sub, DWORD length, BYTE* pdata )
{
	// Transmit frame
	TransmitCommandResponse( SOF_CMD, cmd, sub, length, pdata );
}

// Send response.
void LE9x0Command::TransmitResponse( BYTE cmd, BYTE sub, DWORD length, BYTE* pdata )
{
	// Transmit frame
	TransmitCommandResponse( SOF_RES, cmd, sub, length, pdata );
}

// Send command & Waiting for a response.
void LE9x0Command::TransmitCommandWait( BYTE cmd, BYTE sub, DWORD length, BYTE* pdata, FRAME* pRes )
{
	// Transmit frame
	TransmitCommandResponse( SOF_CMD, cmd, sub, length, pdata );

	// Waiting for a response
	while( !GetResponseFrame( cmd, pRes ) ) { Sleep( 1 ); }
}

// Waiting for a command form LE-9x0.
void LE9x0Command::ReceiveCommandWait( BYTE cmd, FRAME* pRes )
{
	// Waiting for a command
	while( !GetCommandFrame( cmd, pRes ) ) { Sleep( 1 ); }
}

// Convert the AD value of LE-910R to a real value.
// @param[in]	range : Measurement range
// @param[in]	ad : LE-910R analog readings
// @return		Calculated voltage / current / temperature
float LE9x0Command::ConvertAnalogDataFromLE910( RANGE range, DWORD ad )
{
	float tmp = 0;

	switch( range )
	{
	case RANGE_100mV:
		if( ad & 0x800000 )
			tmp = -100 * (float)((~ad + 1) & 0xFFFFFF) / (float)0x7FFFFF;
		else
			tmp = 100 * (float)ad / (float)0x7FFFFF;
		break;
	case RANGE_1V:
		if( ad & 0x800000 )
			tmp = -1 * (float)((~ad + 1) & 0xFFFFFF) / (float)0x7FFFFF;
		else
			tmp = 1 * (float)ad / (float)0x7FFFFF;
		break;
	case RANGE_10V:
		if( ad & 0x800000 )
			tmp = -10 * (float)((~ad + 1) & 0xFFFFFF) / (float)0x7FFFFF;
		else
			tmp = 10 * (float)ad / (float)0x7FFFFF;
		break;
	case RANGE_30V:
		if( ad & 0x800000 )
			tmp = -30 * (float)((~ad + 1) & 0xFFFFFF) / (float)0x7FFFFF;
		else
			tmp = 30 * (float)ad / (float)0x7FFFFF;
		break;

	case RANGE_I_250ohm:
	case RANGE_I_50ohm:
		tmp = 20 * ad / (float)0x7FFFFF;
		break;

	case RANGE_TC:
		if( (ad == 0x800000) || (ad == 0x7FFFFF) )
		{	// Disconnection
			tmp = (float)0x800000;
		}else{
			if( ad & 0x800000 )
				tmp = -1 * (float)(((float)(~ad & 0xFFFFFF) + 1) / 2560.);
			else
				tmp = (float)((float)ad / 2560.);
		}
		break;

	default:
		break;
	}

	return tmp;
}

// Convert the AD value of LE-930R to a real value.
// @param[in]	type : Output type
// @param[in]	ad : LE-930R analog readings
// @return		Calculated voltage / current
float LE9x0Command::ConvertAnalogDataFromLE930( LE930_TYPE type, DWORD ad )
{
	float tmp = 0;

	switch( type )
	{
		case LE930_TYPE_VOLTAGE_100mV:
			if( ad & 0x8000 )
				tmp = -100 * (float)((~ad + 1) & 0xFFFF) / (float)0x7FFF;
			else
				tmp = 100 * (float)ad / (float)0x7FFF;
			break;

		case LE930_TYPE_VOLTAGE_10V:
			if( ad & 0x8000 )
				tmp = -10 * (float)((~ad + 1) & 0xFFFF) / (float)0x7FFF;
			else
				tmp = 10 * (float)ad / (float)0x7FFF;
			break;

		case LE930_TYPE_CURRENT_INT:
		case LE930_TYPE_CURRENT_EXT:
			tmp = 20 * ad / (float)0x7FFF;
			break;
	}

	return tmp;
}

// Convert real values to AD values for LE-930R.
// @param[in]	type : Output type
// @param[in]	val : Voltage or Current value
// @return		AD value to set to LE-930R
DWORD LE9x0Command::ConvertRealDataLE930( LE930_TYPE type, float val )
{
	DWORD ad = 0;

	switch( type )
	{
	case LE930_TYPE_VOLTAGE_100mV:
		if (val >= 0) {
			ad = (DWORD)((0x7FFF * val + 50) / 100);
		}
		else {
			ad = (DWORD)(((0x8000 * (-1 * val) + 50) / 100) - 1);
			ad = ~ad & 0xFFFF;
		}
		break;
	case LE930_TYPE_VOLTAGE_10V:
		if (val >= 0) {
			ad = (DWORD)((0x7FFF * val + 5) / 10);
		}
		else {
			ad = (DWORD)(((0x8000 * (-1 * val) + 5) / 10) - 1);
			ad = ~ad & 0xFFFF;
		}
		break;
	case LE930_TYPE_CURRENT_INT:
	case LE930_TYPE_CURRENT_EXT:
		ad = (DWORD)((0x7FFF * val + 10) / 20);
		break;
	}

	return ad;
}

// Convert real values to AD values for LE-930R.
// @param[in]	type : Output type
// @param[in]	val : Voltage or Current value
// @return		AD value to set to LE-940R
DWORD LE9x0Command::ConvertRealDataLE940( LE940_TYPE type, float val )
{
	DWORD ad = 0;

	switch( type )
	{
	case LE940_TYPE_VOLTAGE_32V:
		if (val >= 0) {
			ad = (DWORD)((0x7FFF * val + 16) / 32);
		}
		else {
			ad = (DWORD)(((0x8000 * (-1 * val) + 16) / 32) - 1);
			ad = ~ad & 0xFFFF;
		}
		break;
	case LE940_TYPE_CURRENT:
		ad = (DWORD)((0x7FFF * val + 10) / 20);
		break;
	}

	return ad;
}

// Add response list.
DWORD LE9x0Command::AddResponseList( BYTE code )
{
	CMDRES_STAT stat;
	FRAME Frame = {0};

	stat.fSet = false;
	stat.bCode = code;
	stat.Frame = Frame;

	ResponseList.push_back( stat );
	return ResponseList.size();
}

// Set frame data in the response list.
void LE9x0Command::SetResponseFrame( BYTE code, FRAME* pframe )
{
	EnterCriticalSection( &mCriticalSection ); 

	for( DWORD i = 0; i < ResponseList.size(); i++ )
	{
		if( ResponseList[i].bCode == code )
		{
			if( !ResponseList[i].fSet )
			{
				ResponseList[i].Frame = *pframe;
				ResponseList[i].fSet = true;
			}
			break;
		}
	}

	LeaveCriticalSection( &mCriticalSection );
}

// Get frame data in the response list.
bool LE9x0Command::GetResponseFrame( BYTE code, FRAME* pframe )
{
	bool ret = false;

	EnterCriticalSection( &mCriticalSection ); 

	for( DWORD i = 0; i < ResponseList.size(); i++ )
	{
		if( ResponseList[i].bCode == code )
		{
			if( ResponseList[i].fSet )
			{
				*pframe = ResponseList[i].Frame;
				ResponseList[i].fSet = false;
				ret = true;
				break;
			}
		}
	}

	LeaveCriticalSection( &mCriticalSection );

	return ret;
}

// Add command list.
DWORD LE9x0Command::AddCommandList( BYTE code )
{
	CMDRES_STAT stat;
	FRAME Frame = {0};

	stat.fSet = false;
	stat.bCode = code;
	stat.Frame = Frame;

	CommandList.push_back( stat );
	return CommandList.size();
}

// Set frame data in the command list.
void LE9x0Command::SetCommandFrame( BYTE code, FRAME* pframe )
{
	EnterCriticalSection( &mCriticalSection ); 

	for( DWORD i = 0; i < CommandList.size(); i++ )
	{
		if( CommandList[i].bCode == code )
		{
			if( !CommandList[i].fSet )
			{
				CommandList[i].Frame = *pframe;
				CommandList[i].fSet = true;
			}
			break;
		}
	}

	LeaveCriticalSection( &mCriticalSection );
}

// Get frame data in the command list.
bool LE9x0Command::GetCommandFrame( BYTE code, FRAME* pframe )
{
	bool ret = false;

	EnterCriticalSection( &mCriticalSection ); 

	for( DWORD i = 0; i < CommandList.size(); i++ )
	{
		if( CommandList[i].bCode == code )
		{
			if( CommandList[i].fSet )
			{
				*pframe = CommandList[i].Frame;
				CommandList[i].fSet = false;
				ret = true;
				break;
			}
		}
	}

	LeaveCriticalSection( &mCriticalSection );

	return ret;
}
